@@ -279,8 +279,8 @@ def numbers_are_equalish(val, ref):
         return {
             "status": "failed",
             "explanation": (
-                "Value {val} does not match expected value {ref} to the"
-                " required precision."
+                f"Value {val} does not match expected value {ref} to the"
+                f" required precision."
             )
         }
 
@@ -2003,13 +2003,15 @@ def make_image_comparator(
         half_diff_fraction = half_diff_area / len(val_pixels)
         diff_score = diff_fraction + half_diff_fraction / 2
 
+        diff_pct = round_to_sig_figs(diff_fraction * 100, 2)
         diff_msg = (
-            f"{diff_fraction * 100:.1f}% of pixels were"
+            f"{diff_pct}% of pixels were"
             f" different"
         )
         if half_diff_fraction > 0:
+            half_pct = round_to_sig_figs(half_diff_fraction * 100, 2)
             diff_msg += (
-                f" and {half_diff_fraction * 100:.1f}% of pixels were"
+                f" and {half_pct}% of pixels were"
                 f" somewhat different"
             )
 
