@@ -578,9 +578,11 @@ def dynamic_html_repr(thing, reasonable=200, limit=10000):
     """
     rep = big_repr(thing)
     if len(rep) <= reasonable:
-        return '<pre class="short_repr">{rep}</pre>'.format(rep=rep)
+        return '<pre class="short_repr">{rep}</pre>'.format(
+            rep=escape(rep)
+        )
     else:
-        return build_display_box(truncate(rep, limit))
+        return build_display_box(escape(truncate(rep, limit)))
 
 
 #-------------------------------#
