@@ -20,16 +20,3 @@ def whatever(a, b):
         return "B"
     else:
         raise ValueError("Crash!")
-
-
-# Expectations
-
-import optimism as opt
-
-
-# Not enough expectations (3 required)
-opt.testCase(whatever(True, True))
-opt.expectResult("Both")
-# Expectation fails (without crashing):
-opt.testCase(whatever(True, False))
-opt.expectResult("B")
