@@ -57,9 +57,9 @@ meta.example("perfect")
 
 meta.example("imperfect")
 
-meta.expect("partial", "procedure", "core", "define")
-meta.expect("partial", "product", "core", "processData")
-meta.expect("partial", "behavior", "extra", "run")
+meta.expect("partial", "define")
+meta.expect("partial", "processData must")
+meta.expect("partial", "run")
 # Note: the submission fails all of the extra tests, but output is
 # similar enough to count as partial success each time
 
