@@ -135,7 +135,7 @@ function validate_task_submit(task_form) {
     // Takes a task form DOM element and runs validation to
     // disable/enable the submit button and add/remove relevant messages.
 
-    let psid = task_form.getAttribute("data-pset");
+    let prid = task_form.getAttribute("data-project");
     let taskid = task_form.getAttribute("data-task");
 
     let file_browser = task_form.querySelector('input[name="upload"]');
@@ -146,7 +146,7 @@ function validate_task_submit(task_form) {
     // is finalized, skip validation, remove messages, and enable the
     // upload button.
     let task_item = task_form.parentElement.parentElement;
-    let pset_item = task_item.parentElement.parentElement;
+    let project_item = task_item.parentElement.parentElement;
 
     let unsubmitted = task_item.classList.contains("unsubmitted");
 
@@ -154,7 +154,7 @@ function validate_task_submit(task_form) {
     if (
         validation_control == null
      || !validation_control.checked
-     || pset_item.classList.contains("status-final")
+     || project_item.classList.contains("status-final")
     ) {
         set_validation_messages(task_form, {});
         submit_button.removeAttribute("disabled");
@@ -214,8 +214,8 @@ function validate_task_submit(task_form) {
     set_validation_messages(task_form, messages);
 
     // Enable/disable the upload button
-    let file_desc_id = psid + '-' + taskid + '-filename';
-    let time_desc_id = psid + '-' + taskid + '-time_spent';
+    let file_desc_id = prid + '-' + taskid + '-filename';
+    let time_desc_id = prid + '-' + taskid + '-time_spent';
     if (Object.keys(messages).length > 0) {
         submit_button.setAttribute("disabled", true);
         submit_button.classList.add("invalid");
@@ -233,7 +233,7 @@ function set_validation_messages(task_form, messages) {
     // Removes any old validation messages for a form and adds new ones.
     // Messages should be a mapping from topics to HTML message strings.
 
-    let psid = task_form.getAttribute("data-pset");
+    let prid = task_form.getAttribute("data-project");
     let taskid = task_form.getAttribute("data-task");
 
     let task_item = task_form.parentElement.parentElement;
@@ -249,7 +249,7 @@ function set_validation_messages(task_form, messages) {
         let msg = document.createElement("li");
         msg.classList.add("validation");
         msg.classList.add("topic-" + msg_topic);
-        msg.setAttribute("id", psid + '-' + taskid + '-' + msg_topic);
+        msg.setAttribute("id", prid + '-' + taskid + '-' + msg_topic);
         msg.innerHTML = messages[msg_topic];
         msg_list.appendChild(msg);
     }
