@@ -8,7 +8,7 @@ storage.py
 # Attempts at 2/3 dual compatibility:
 from __future__ import print_function
 
-__version__ = "0.2"
+__version__ = "0.3.1"
 
 import sys, os, shutil, subprocess, threading, copy
 import time, datetime
@@ -179,7 +179,7 @@ def concepts_file(course, semester):
     """
     return os.path.join(
         evaluation_directory(course, semester),
-        _CONFIG.get("CONCEPTS_FILE", "concepts.json")
+        _CONFIG.get("CONCEPTS_FILE", "pl_concepts.json")
     )
 
 
@@ -224,9 +224,9 @@ def redis_key_suffix(key):
     return key[len(redis_key("")):]
 
 
-def inflight_key(course, semester, username, pset, task, phase):
+def inflight_key(course, semester, username, project, task, phase):
     """
-    The in-flight key for a class/semester/user/pset/task/phase.
+    The in-flight key for a class/semester/user/project/task/phase.
     """
     return redis_key(
         ':'.join(
@@ -235,7 +235,7 @@ def inflight_key(course, semester, username, pset, task, phase):
                 semester,
                 "inflight",
                 username,
-                pset,
+                project,
                 task,
                 phase
             ]
@@ -243,36 +243,36 @@ def inflight_key(course, semester, username, pset, task, phase):
     )
 
 
-def extension_key(course, semester, username, pset, phase):
+def extension_key(course, semester, username, project, phase):
     """
     The Redis key for the extension for a
-    class/semester/user/pset/phase.
+    class/semester/user/project/phase.
     """
     return redis_key(
-        ':'.join([course, semester, "ext", username, pset, phase])
+        ':'.join([course, semester, "ext", username, project, phase])
     )
 
 
-def time_spent_key(course, semester, username, pset, phase, task):
+def time_spent_key(course, semester, username, project, phase, task):
     """
     The Redis key for the time-spent info for a
-    class/semester/user/pset/phase/task.
+    class/semester/user/project/phase/task.
     """
     return redis_key(
         ':'.join(
-            [course, semester, "spent", username, pset, phase, task]
+            [course, semester, "spent", username, project, phase, task]
         )
     )
 
 
-def evaluation_key(course, semester, username, pset, phase, task):
+def evaluation_key(course, semester, username, project, phase, task):
     """
     The Redis key for the custom evaluation info for a
-    class/semester/user/pset/phase/task.
+    class/semester/user/project/phase/task.
     """
     return redis_key(
         ':'.join(
-            [course, semester, "eval", username, pset, phase, task]
+            [course, semester, "eval", username, project, phase, task]
         )
     )
 
@@ -287,6 +287,7 @@ def old_exercise_key(course, semester, username, exercise):
         )
     )
 
+
 def exercise_key(course, semester, username, exercise, category):
     """
     The Redis key for the outcomes-list-history for a particular
@@ -473,37 +474,40 @@ def get_task_info(course, semester):
         return None
 
     # Augment task info
-    psfmt = _CONFIG.get("PSET_URLS", {}).get(course, "#")
-    taskfmt = _CONFIG.get("TASK_URLS", {}).get(course, "#")
-    for pset in result["psets"]:
-        pset["url"] = psfmt.format(
+    prfmt = result.get(
+        "project_url_format",
+        _CONFIG.get("DEFAULT_PROJECT_URL_FORMAT", "#")
+    )
+    taskfmt = result.get(
+        "task_url_format",
+        _CONFIG.get("DEFAULT_TASK_URL_FORMAT", "#")
+    )
+    for project in result.get("projects", result.get("psets")):
+        project["url"] = prfmt.format(
             semester=semester,
-            pset=pset["id"]
+            project=project["id"]
         )
-        for task in pset["tasks"]:
+        for task in project["tasks"]:
             task["url"] = taskfmt.format(
                 semester=semester,
-                pset=pset["id"],
+                project=project["id"],
                 task=task["id"]
             )
-            # Graft static task info into pset task entry
+            # Graft static task info into project task entry
             task.update(result["tasks"][task["id"]])
 
     # Augment exercise info if it's present
-    exfmtmap = _CONFIG.get("EXERCISE_URLS", {}).get(course, {})
+    exfmt = result.get(
+        "exercise_url_format",
+        _CONFIG.get("DEFAULT_EXERCISE_URL_FORMAT", "#")
+    )
     if 'exercises' in result:
         for egroup in result["exercises"]:
             if 'url' not in egroup:
-                for prefix in exfmtmap:
-                    if egroup["group"].startswith(prefix):
-                        egroup["url"] = exfmtmap[prefix].format(
-                            semester=semester,
-                            group=egroup["group"]
-                        )
-
-            # No prefix matches perhaps
-            if 'url' not in egroup:
-                egroup["url"] = '#'
+                egroup["url"] = exfmt.format(
+                    semester=semester,
+                    group=egroup["group"]
+                )
 
     return result
 
@@ -582,14 +586,14 @@ def get_student_info(course, semester):
     )
 
 
-def get_extension(course, semester, username, pset, phase):
+def get_extension(course, semester, username, project, phase):
     """
     Gets the extension value (as an integer number in hours) for a user
-    on a given phase of a given problem set. Returns 0 if there is no
+    on a given phase of a given project. Returns 0 if there is no
     extension info for that user. Returns None if there's an error
     reading the value.
     """
-    key = extension_key(course, semester, username, pset, phase)
+    key = extension_key(course, semester, username, project, phase)
     try:
         result = _REDIS.get(key)
     except Exception:
@@ -618,17 +622,17 @@ def set_extension(
     course,
     semester,
     username,
-    psid,
+    prid,
     phase,
     duration=True,
     only_from=None
 ):
     """
     Sets an extension value for the given user on the given phase of the
-    given pset (in the given course/semester). May be an integer number
-    of hours, or just True (the default) for the standard extension
-    (whatever is listed in tasks.json). Set to False to remove any
-    previously granted extension.
+    given project (in the given course/semester). May be an integer
+    number of hours, or just True (the default) for the standard
+    extension (whatever is listed in tasks.json). Set to False to remove
+    any previously granted extension.
 
     If only_from is provided, the operation will fail when the extension
     value being updated isn't set to that value (may be a number of
@@ -636,7 +640,7 @@ def set_extension(
     that case, this function will return False if it fails. Set
     only_from to None to unconditionally update the extension.
     """
-    key = extension_key(course, semester, username, psid, phase)
+    key = extension_key(course, semester, username, prid, phase)
     task_info = get_task_info(course, semester)
     ext_hours = task_info.get("extension_hours", 24)
 
@@ -729,12 +733,12 @@ def get_inflight(
     semester,
     username,
     phase,
-    psid,
+    prid,
     taskid
 ):
     """
     Returns a quadruple containing the timestamp at which processing for
-    the given user/phase/pset/task was started, the filename of the log
+    the given user/phase/project/task was started, the filename of the log
     file for that evaluation run, the filename of the report file that
     will be generated when it's done, and a string indicating the status
     of the run. Reads that log file to check whether the process has
@@ -759,7 +763,7 @@ def get_inflight(
     file even after a second attempt, the timestamp will be set to
     "error" with status and filename values of None.
     """
-    key = inflight_key(course, semester, username, phase, psid, taskid)
+    key = inflight_key(course, semester, username, phase, prid, taskid)
 
     try:
         response = _REDIS.lrange(key, 0, -1)
@@ -867,17 +871,17 @@ def get_inflight(
     return result
 
 
-def put_inflight(course, semester, username, phase, psid, taskid):
+def put_inflight(course, semester, username, phase, prid, taskid):
     """
-    Picks new log and report filenames for the given user/phase/pset/task
-    and returns a quad containing a string timestamp, the new log
-    filename, the new report filename, and the status string "initial",
-    while also writing that information into the inflight data for that
-    user so that get_inflight will return it until evaluation is
-    finished.
+    Picks new log and report filenames for the given
+    user/phase/project/task and returns a quad containing a string
+    timestamp, the new log filename, the new report filename, and the
+    status string "initial", while also writing that information into
+    the inflight data for that user so that get_inflight will return it
+    until evaluation is finished.
 
     Returns (None, None, None, None) if there is already an in-flight
-    log file for this user/pset/task that has a status other than
+    log file for this user/project/task that has a status other than
     "error", "expired", or "completed".
 
     Returns ("error", None, None, None) if it encounters a situation
@@ -887,7 +891,7 @@ def put_inflight(course, semester, username, phase, psid, taskid):
     against race conditions on the log and report filenames.
     """
     # The Redis key for inflight info
-    key = inflight_key(course, semester, username, phase, psid, taskid)
+    key = inflight_key(course, semester, username, phase, prid, taskid)
 
     with _REDIS.pipeline() as pipe:
         try:
@@ -905,9 +909,9 @@ def put_inflight(course, semester, username, phase, psid, taskid):
             timestamp = potluck.time_utils.timestring()
 
             # Get unused log and report filenames
-            istring = "{phase}-{psid}-{taskid}-{timestamp}".format(
+            istring = "{phase}-{prid}-{taskid}-{timestamp}".format(
                 phase=phase,
-                psid=psid,
+                prid=prid,
                 taskid=taskid,
                 timestamp=timestamp
             )
@@ -976,13 +980,13 @@ def put_inflight(course, semester, username, phase, psid, taskid):
     return ifinfo
 
 
-def fetch_time_spent(course, semester, username, phase, psid, taskid):
+def fetch_time_spent(course, semester, username, phase, prid, taskid):
     """
-    Returns a time-spent record for the given user/phase/pset/task. It
-    has the following keys:
+    Returns a time-spent record for the given user/phase/project/task.
+    It has the following keys:
 
     - "phase": The phase (a string).
-    - "psid": The problem-set ID (a string).
+    - "prid": The project ID (a string).
     - "taskid": The task ID (a string).
     - "updated_at": A timestring (see `potluck.time_utils.timestring`)
         indicating when the information was last updated.
@@ -995,8 +999,8 @@ def fetch_time_spent(course, semester, username, phase, psid, taskid):
         entry was entered.
         TODO: preserve across multiple empty entries?
 
-    Returns None if there is no information for that user/pset/task yet,
-    or if an error is encountered while trying to access that
+    Returns None if there is no information for that user/project/task
+    yet, or if an error is encountered while trying to access that
     information.
     """
     # Redis key to use
@@ -1004,7 +1008,7 @@ def fetch_time_spent(course, semester, username, phase, psid, taskid):
         course,
         semester,
         username,
-        psid,
+        prid,
         phase,
         taskid
     )
@@ -1039,7 +1043,7 @@ def fetch_time_spent(course, semester, username, phase, psid, taskid):
 
     result = {
         "phase": phase,
-        "psid": psid,
+        "prid": prid,
         "taskid": taskid,
         "updated_at": response[0],
         "time_spent": spent
@@ -1056,7 +1060,7 @@ def record_time_spent(
     semester,
     username,
     phase,
-    psid,
+    prid,
     taskid,
     time_spent
 ):
@@ -1074,7 +1078,7 @@ def record_time_spent(
         course,
         semester,
         username,
-        psid,
+        prid,
         phase,
         taskid
     )
@@ -1150,14 +1154,14 @@ def record_time_spent(
         )
 
 
-def fetch_evaluation(course, semester, username, phase, psid, taskid):
+def fetch_evaluation(course, semester, username, phase, prid, taskid):
     """
     Fetches the manual evaluation information for the given
-    user/phase/pset/task. The result will be a dictionary with the
+    user/phase/project/task. The result will be a dictionary with the
     following keys:
 
     - "phase": The phase (a string).
-    - "psid": The problem-set ID (a string).
+    - "prid": The project ID (a string).
     - "taskid": The task ID (a string).
     - "updated_at": A timestring (see `potluck.time_utils.timestring`)
         indicating when the information was last updated.
@@ -1170,15 +1174,15 @@ def fetch_evaluation(course, semester, username, phase, psid, taskid):
         override, which should always be the case for non-initial phases.
 
     Returns None instead of a dictionary if there is no information for
-    that user/pset/task yet, or if an error is encountered while trying
-    to access that information.
+    that user/project/task yet, or if an error is encountered while
+    trying to access that information.
     """
     # Redis key to use
     key = evaluation_key(
         course,
         semester,
         username,
-        psid,
+        prid,
         phase,
         taskid
     )
@@ -1223,7 +1227,7 @@ def fetch_evaluation(course, semester, username, phase, psid, taskid):
 
     result = {
         "phase": phase,
-        "psid": psid,
+        "prid": prid,
         "taskid": taskid,
         "updated_at": response[0],
         "notes": response[1],
@@ -1239,7 +1243,7 @@ def set_evaluation(
     semester,
     username,
     phase,
-    psid,
+    prid,
     taskid,
     notes,
     override="",
@@ -1247,7 +1251,7 @@ def set_evaluation(
 ):
     """
     Updates the custom evaluation info for a particular task submitted by
-    a particular use for a certain phase of a specific pset (in a
+    a particular use for a certain phase of a specific project (in a
     course/semester).
 
     Completely erases the previous custom evaluation info.
@@ -1274,7 +1278,7 @@ def set_evaluation(
         course,
         semester,
         username,
-        psid,
+        prid,
         phase,
         taskid
     )
@@ -1400,6 +1404,7 @@ def fetch_old_outcomes(course, semester, username, exercise):
 
     return info
 
+
 def fetch_outcomes(course, semester, username, exercise, category):
     """
     Fetches the outcomes-list information for the given
@@ -1471,6 +1476,34 @@ def fetch_outcomes(course, semester, username, exercise, category):
     return info
 
 
+def update_submission_credit(submission, deadline, late_fraction):
+    """
+    Helper for updating submission group credit based on timeliness
+    given the specified `deadline`. Sets the "on_time" and
+    "group_credit" slots of the submission. "on_time" is based on the
+    "submitted_at" slot and the specified `deadline`; "group_credit" is
+    based on the original "group_credit" value (or "credit" value if
+    there is no "group_credit" value) and is multiplied by the
+    `late_fraction` if the submission is not on-time.
+
+    Modifies the given submission dictionary; does not return anything.
+    """
+    when = potluck.time_utils.time_from_timestring(
+        submission["submitted_at"]
+    )
+    submission["group_credit"] = (
+        submission.get(
+            "group_credit",
+            submission.get("credit", 0)
+        )
+    ) or 0  # this ensures None becomes 0 if there's an explicit None
+    if when <= deadline:
+        submission["on_time"] = True
+    else:
+        submission["on_time"] = False
+        submission["group_credit"] *= late_fraction
+
+
 def fetch_best_outcomes(
     course,
     semester,
@@ -1517,21 +1550,7 @@ def fetch_best_outcomes(
     if fulls is None:
         fulls = []
     for submission in reversed(fulls):  # iterate backwards chronologically
-        when = potluck.time_utils.time_from_timestring(
-            submission["submitted_at"]
-        )
-        if when <= deadline:
-            submission["on_time"] = True
-            submission["group_credit"] = submission.get(
-                "group_credit",
-                submission.get("credit", 0)
-            )
-        else:
-            submission["on_time"] = False
-            submission["group_credit"] = late_fraction * submission.get(
-                "group_credit",
-                submission.get("credit", 0)
-            )
+        update_submission_credit(submission, deadline, late_fraction)
 
         if best is None or submission["group_credit"] >= best["group_credit"]:
             best = submission
@@ -1552,21 +1571,7 @@ def fetch_best_outcomes(
         if partials is None:
             partials = []
         for submission in partials:
-            when = potluck.time_utils.time_from_timestring(
-                submission["submitted_at"]
-            )
-            if when <= deadline:
-                submission["on_time"] = True
-                submission["group_credit"] = submission.get(
-                    "group_credit",
-                    submission.get("credit", 0)
-                )
-            else:
-                submission["on_time"] = False
-                submission["group_credit"] = late_fraction * submission.get(
-                    "group_credit",
-                    submission.get("credit", 0)
-                )
+            update_submission_credit(submission, deadline, late_fraction)
 
             if (
                 best is None
@@ -1587,22 +1592,8 @@ def fetch_best_outcomes(
         if nones is not None:
             # Always take chronologically last one; none of these are worth
             # any credit anyways.
-            submission = nones[-1]
-            when = potluck.time_utils.time_from_timestring(
-                submission["submitted_at"]
-            )
-            if when <= deadline:
-                submission["on_time"] = True
-                submission["group_credit"] = submission.get(
-                    "group_credit",
-                    submission.get("credit", 0)
-                )
-            else:
-                submission["on_time"] = False
-                submission["group_credit"] = late_fraction * submission.get(
-                    "group_credit",
-                    submission.get("credit", 0)
-                )
+            best = nones[-1]
+            update_submission_credit(best, deadline, late_fraction)
 
     # Try legacy info
     if best is None or best["group_credit"] < 1:
@@ -1620,21 +1611,7 @@ def fetch_best_outcomes(
             if old_on_time is False:
                 submission["group_credit"] /= late_fraction
 
-            when = potluck.time_utils.time_from_timestring(
-                submission["submitted_at"]
-            )
-            if when <= deadline:
-                submission["on_time"] = True
-                submission["group_credit"] = submission.get(
-                    "group_credit",
-                    submission.get("credit", 0)
-                )
-            else:
-                submission["on_time"] = False
-                submission["group_credit"] = late_fraction * submission.get(
-                    "group_credit",
-                    submission.get("credit", 0)
-                )
+            update_submission_credit(submission, deadline, late_fraction)
 
             if (
                 best is None
@@ -1775,12 +1752,12 @@ def get_feedback_summary(
     task_info,
     username,
     phase,
-    psid,
+    prid,
     taskid
 ):
     """
     This retrieves just the feedback summary information that appears on
-    the dashboard for a given user/phase/pset/task. That much info is
+    the dashboard for a given user/phase/project/task. That much info is
     light enough to cache, so we do cache it to prevent hitting the disk
     a lot for each dashboard view.
     """
@@ -1789,7 +1766,7 @@ def get_feedback_summary(
         semester,
         username,
         phase,
-        psid,
+        prid,
         taskid
     )
     fallback = default_feedback_summary()
@@ -1813,13 +1790,14 @@ def get_feedback(
     task_info,
     username,
     phase,
-    psid,
+    prid,
     taskid
 ):
     """
     Gets feedback for the user's latest pre-deadline submission for the
-    given phase/pset/task. Instead of caching these values (which would
-    be expensive memory-wise over time) we hit the disk every time.
+    given phase/project/task. Instead of caching these values (which
+    would be expensive memory-wise over time) we hit the disk every
+    time.
 
     Returns a dictionary with at least a 'status' entry. This will be
     'ok' if the report was read successfully, or 'missing' if the report
@@ -1838,7 +1816,7 @@ def get_feedback(
         semester,
         username,
         phase,
-        psid,
+        prid,
         taskid
     )
     if ts is None: # No submission
@@ -1911,12 +1889,12 @@ def get_feedback_html(
     task_info,
     username,
     phase,
-    psid,
+    prid,
     taskid
 ):
     """
     Gets feedback for the user's latest pre-deadline submission for the
-    given phase/pset/task, as html instead of as json (see
+    given phase/project/task, as html instead of as json (see
     `get_feedback`). Instead of caching these values (which would be
     expensive memory-wise over time) we hit the disk every time.
 
@@ -1932,7 +1910,7 @@ def get_feedback_html(
         semester,
         username,
         phase,
-        psid,
+        prid,
         taskid
     )
     if ts is None: # No submission
